package brownstrategy;

import game.Game;
import game.Strategy;
import game.Utility;
import util.Constants;
import java.util.Random;

import brownstrategy.agent.NondeterministicAgent;

import strategy.ExternalRegretMatchingStrategy;
import strategy.GIGA;
import strategy.InternalRegretMatchingStrategy;


/**
 * An abstract class for strategies which depend only upon
 * the utility of the action taken and the utility the player 
 * would have gotten had it taken another strategy.
 * 
 * @see ExternalRegretMatchingStrategy
 * @see InternalRegretMatchingStrategy
 * @see GIGA
 * 
 * This class also provides a random number generator. It is unclear
 * if this is the best place for it.
 * @author maz
 * 
 * edited by jberg
 *
 */
public abstract class BrownUtilityBasedStrategy extends NondeterministicAgent {
	protected int numActions;
	protected Random r;

	/**
	 * Note that a strategy must implement
	 * a constructor with (Game, long, String)
	 * @param g
	 * @param seed
	 */
	public BrownUtilityBasedStrategy(Game g, long seed, String options) {
		super(g,seed,options);
		this.numActions = g.getNumActions();
		this.r = new Random(seed);
	}

	public int[] getPossibleUtilities(int[] actions){
		int[] possibleUtilities=new int[numActions];
		for(int i=0;i<possibleUtilities.length;i++){
			int[] newactions = actions.clone();
			newactions[_playerIdx]=i;
			possibleUtilities[i]=Constants.payoffs[newactions[0]][newactions[1]][newactions[2]][_playerIdx];
		}
		return possibleUtilities;
	}

	public void observeOutcome(int[] actions){
		_day++;
		int[] newactions = actions.clone();
		int[] possibleUtilities = getPossibleUtilities(newactions);
		int currentAction = newactions[_playerIdx];
		observeUtilities(possibleUtilities,currentAction);
	}

	/**
	 * When observeOutcome() is called, the utilities are calculated
	 * and passed as an array.
	 * @param possibleUtilities the utilities of all possible actions.
	 * @param currentAction 
	 */
	public abstract void observeUtilities(int[] possibleUtilities, int currentAction);
}
