package brownstrategy;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Stack;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;
import brownstrategy.agent.NondeterministicAgent;

/**
 * 
 * This agent plays by multiplying everyones
 * moves from the previous round mod 12
 * 
 * @author jberg
 *
 */
public class MultiplyAllStrategy extends NondeterministicAgent {
	/**
	 * The number of actions available in the game.
	 */
	int numActions;
	
	/**
	 * The last actions that were observed
	 */
	int[] lastActions;

	/**
	 * A random number generator for the random actions.
	 */
	Random r;

	private Stack<int[]> lastActionsStack;
	
	public MultiplyAllStrategy(Game g, long seed, String options) {
		super(g,seed,options);
		this.numActions = g.getNumActions();
		this.lastActions = null;
		this.r = new Random(seed);
		lastActionsStack = new Stack<int[]>();
	}


	@Override
	public Agent getCopy() {
		return new MultiplyAllStrategy(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new MultiplyAllStrategy(g, seed, options);
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		int action;
		if(lastActions != null) {
			for(int i = 0; i < 3; i++) {
				if(lastActions[i] == 0) {
					lastActions[i] = 1;
				}
			}
			action = (lastActions[0] * lastActions[1] * lastActions[2]) % 12;
		}
		else {
			action = r.nextInt(numActions);
		}
		_cacheDist[action] = 1;
		return _cacheDist;
	}

	@Override
	public void observeOutcome(int[] actions) {
		lastActions = actions.clone();
		_day++;
	}
	
	@Override
	public void lookAhead(int[] actions) {
		lastActionsStack.push(lastActions);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		lastActions = lastActionsStack.pop();
		_day--;
	}

}
