package brownstrategy;

import java.util.ArrayList;
import java.util.StringTokenizer;

import util.Constants;
import game.Game;
import brownstrategy.agent.Agent;
import brownstrategy.agent.ModelBasedAgent;
import brownstrategy.oracles.UniformDistributionOracle;

public class SimpleBestResponse extends ModelBasedAgent {


	private int _numIters;
	private int _secondGuess;

	public SimpleBestResponse(Game g, long seed, String options) {
		super(g, seed, options);
		_numIters = 0;
		_secondGuess = 2;
		StringTokenizer st = new StringTokenizer(options, ",");
		if (st.hasMoreTokens()) {
			st.nextToken();
			if (st.hasMoreTokens()) {
				_numIters = Integer.parseInt(st.nextToken());
				if (st.hasMoreTokens()) {
					_secondGuess = Integer.parseInt(st.nextToken());
				}
			}
		}
	}


	@Override
	public double[] getActionDistribution() {

		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;

		//Get expected scores after making each action, given opponent distributions. 
		//[action][player]
		ArrayList<Integer> bestActions;
		//if (METHOD == OVERALL_SCORE) 
		bestActions = getBestActionsByOverallScore(predictor1.getActionDistribution(), predictor2.getActionDistribution());

		//Randomly play any action that has the maximum score
		//(OR Randomly play any action that maximizes the distance between you and the 1st (or next highest) player)
		double[] actionDistribution = new double[Constants.NUM_CLOCK_POSITIONS];
		for (Integer action : bestActions) {
			actionDistribution[action] = 1.0 / bestActions.size();
		}

		ArrayList<Integer> opp1BR=null,opp2BR=null;
		double[] opp1BRarr = predictor1.getActionDistribution();
		double[] opp2BRarr = predictor2.getActionDistribution();
		for(int i = 0; i < _numIters; i++) {
			if(_secondGuess == 0 || _secondGuess == 2) {
				opp1BR = getBestActionsByOverallScore(actionDistribution, opp2BRarr);
			}

			if(_secondGuess == 1 || _secondGuess == 2) {
				opp2BR = getBestActionsByOverallScore(actionDistribution, opp1BRarr);
			}

			if(_secondGuess == 0 || _secondGuess == 2) {
				opp1BRarr = new double[actionDistribution.length];
				for (Integer action : opp1BR) {
					opp1BRarr[action] = 1.0 / opp1BR.size();
				}
			}

			if(_secondGuess == 1 || _secondGuess == 2) {
				opp2BRarr = new double[actionDistribution.length];
				for (Integer action : opp2BR) {
					opp2BRarr[action] = 1.0 / opp2BR.size();
				}
			}

			bestActions = getBestActionsByOverallScore(opp1BRarr, opp2BRarr);
		}

		actionDistribution = new double[Constants.NUM_CLOCK_POSITIONS];
		for (Integer action : bestActions) {
			actionDistribution[action] = 1.0 / bestActions.size();
		}

		_cacheDist = actionDistribution;

		return actionDistribution;
	}


	private ArrayList<Integer> getBestActionsByOverallScore(double[] p1Distribution, double[] p2Distribution) {

		double[][] expectedScoresForPlayers = getExpectedScoresGivenActionDistributions(p1Distribution, p2Distribution);

		//Choose the action that gives you the highest score.
		//TODO: Do we want to maximize our score, or the distance between us and the guy below us? or minimize the distance between us and the guy in 1st?
		//TODO: We should use Jordan's tiebreakers (or some modified version of them) to set the action distribution.
		double bestScore = 0;
		for (int action=0; action<Constants.NUM_CLOCK_POSITIONS; action++) {
			if (expectedScoresForPlayers[action][0] > bestScore) {
				bestScore = expectedScoresForPlayers[action][0];
			}
		}

		ArrayList<Integer> bestActions = new ArrayList<Integer>();
		for (int action=0; action<Constants.NUM_CLOCK_POSITIONS; action++) {
			if (expectedScoresForPlayers[action][0] == bestScore) {
				bestActions.add(new Integer(action));
			}
		}

		return bestActions;
	}


	@Override
	public Agent getCopy() {
		return new SimpleBestResponse(_g, _seed, _options);
	}


	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new SimpleBestResponse(g, seed, options);
	}

	@Override
	public void lookAhead(int[] actions) {
		//as long as the predictors are updated
		//we shouldn't need to do anything
	}

	@Override
	public void lookBack() {
		//as long as the predictors are updated
		//we shouldn't need to do anything
	}


}
