package brownstrategy.agent;

import game.Game;
import game.Strategy;

import java.util.Random;


import util.Constants;


/**
 * A strategy for a player. All players view the game as if
 * they are player 0. A class implementing the strategy
 * type should also implement a constructor of the form
 * MyClass(game.Game game, long randomSeed, String options).
 * 
 * The agent must return a distribution of actions.
 * 
 * @author eric
 *
 */
public abstract class DeterministicAgent extends Agent {

	public Random random;
	
	
	public DeterministicAgent(Game g, long seed, String options) {
		super(g, seed, options);
		random = new Random(seed);
	}
	
	
	public abstract int getAction();
	public abstract void observeOutcome(int[] actions);

	
	/**
	 * Returns distribution of actions that should be probabilistically taken.
	 * In deterministic agent, one action will have probability 1.
	 * @return
	 */
	public double[] getActionDistribution() {
		double[] distribution = new double[Constants.NUM_CLOCK_POSITIONS];
		distribution[getAction()] = 1;
		return distribution;
	}
		
}
