package brownstrategy.ensembles;

import game.Game;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import util.Constants;
import brownstrategy.agent.Agent;
import brownstrategy.agent.NondeterministicAgent;


/**
 *
 *	This is an abstract class for all ensembles
 * 
 * @author jberg
 *
 */

public abstract class AbstractEnsemble extends NondeterministicAgent {
	
	Stack<HashMap<Agent,double[]>> _lastPredictionStack;
	Stack<HashMap<Agent,Double>> _weightstack;

	
	HashMap<Agent,double[]> _lastPredictions;
	HashMap<Agent,Double> _weights;
	
	ArrayList<Agent> _agentList;

	public AbstractEnsemble(Game g, long seed, String options) {
		super(g,seed,options);
		_lastPredictions = new HashMap<Agent, double[]>();
		_weights = new HashMap<Agent, Double>();
		_lastPredictionStack = new Stack<HashMap<Agent,double[]>>();
		_weightstack = new Stack<HashMap<Agent,Double>>();
		_agentList = new ArrayList<Agent>();
	}
	
	public void addAllModels(ArrayList<Agent> models) {
		for(Agent model : models) {
			addModel(model);
		}
	}
	
	public void removeAllModels(ArrayList<Agent> models) {
		for(Agent model : models) {
			removeModel(model);
		}
	}
		
	public void addModel(Agent model) {
		_lastPredictions.put(model,null);
		_weights.put(model, 0.0);
		_agentList.add(model);
	}
	
	public void removeModel(Agent model) {
		_lastPredictions.remove(model);
		_weights.remove(model);
		_agentList.remove(model);
	}
	
	@Override
	public void lookAhead(int[] actions) {
		_lastPredictionStack.push(_lastPredictions);
		_weightstack.push(_weights);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		_lastPredictions = _lastPredictionStack.pop();
		_weights = _weightstack.pop();
	}
	
	@Override
	public double[] getActionDistribution() {
		double[] dist = new double[Constants.NUM_CLOCK_POSITIONS];
		if(_weights.size() > 0) {
			for(Agent agent : _weights.keySet()) {
				double[] tmpDist = _lastPredictions.get(agent);
				if(tmpDist == null) {
					continue;
				}
				double weight = _weights.get(agent);
				for(int i = 0; i < Constants.NUM_CLOCK_POSITIONS; i++) {
					dist[i] = dist[i] + weight * tmpDist[i];
				}
			}
			return normalize(dist);
		}
		double uniform = 1.0/Constants.NUM_CLOCK_POSITIONS;
		for(int i = 0; i < Constants.NUM_CLOCK_POSITIONS; i++) {
			dist[i] = uniform;
		}
		return dist;
	}
}