package brownstrategy.ensembles;

import game.Game;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;

import brownstrategy.agent.Agent;
import brownstrategy.agent.NondeterministicAgent;
import brownstrategy.oracles.PrevSeqIndividualWithUtilOracle;
import brownstrategy.oracles.PrevSeqOpponentsWithUtilOracle;
import brownstrategy.oracles.PrevSequenceAllOracle;
import brownstrategy.oracles.PrevSequenceIndividualOracle;
import brownstrategy.oracles.PrevSequenceAllRotSymOracle;
import brownstrategy.oracles.PrevSequenceOppSymOracle;
import brownstrategy.oracles.PrevSequenceOpponentsOracle;
import brownstrategy.oracles.SameAsLastTimeOracle;

/**
 *
 *	This is an ensemble class of oracles
 * 
 * @author jberg
 *
 */

public class NeuralNetworkMixing extends AbstractEnsemble {

	private double _learningRate;

	public NeuralNetworkMixing(Game g, long seed, String options) {
		super(g,seed,options);
		StringTokenizer st = new StringTokenizer(options, ",");
		if (st.countTokens() >= 2) {
			st.nextToken();
			_learningRate = Double.parseDouble(st.nextToken());
		}
		else {
			_learningRate = .01;
		}
	}

	@Override
	public void observeOutcome(int[] actions) {
		int action = actions[_playerIdx];
		double sum = 0;
		HashMap<Agent,Double> stretchedProbs = new HashMap<Agent, Double>();
		for(Agent agent : _lastPredictions.keySet()) {
			double[] lastPrediction = _lastPredictions.get(agent);
			if(lastPrediction != null) {
				double stretchedProb ;
				if(lastPrediction[action] == 0) {
					stretchedProb = Math.log(.01/(1.0-.01));
				}
				else if(lastPrediction[action] == 1) {
					stretchedProb = Math.log(.99/(1.0-.99));
				}
				else {
					stretchedProb = Math.log(lastPrediction[action]/(1.0-lastPrediction[action]));
				}
				stretchedProbs.put(agent, stretchedProb);
				sum += _weights.get(agent) * stretchedProb;
			}
		}

//		System.out.print("Sum: " + sum);
		sum = 1.0 / (1.0 + Math.exp(-sum));
//		System.out.print(", Sum2: " + sum);
		double[] ensemblePredictions = this.getActionDistribution();
		int pred = isMax(ensemblePredictions[action],ensemblePredictions);
//		System.out.print(", pred: " + pred + "\n");
		for(Agent agent : _lastPredictions.keySet()) {
			double[] lastPrediction = _lastPredictions.get(agent);
			if(lastPrediction != null) {
				double weight = Math.max(_weights.get(agent) + _learningRate  * stretchedProbs.get(agent) * (pred-sum),0.0);
				_weights.put(agent,weight);
			}
		}
		
		/*
		 * Get new predictions
		 */
		for(Agent agent : _lastPredictions.keySet()) {
			_lastPredictions.put(agent, agent.getActionDistribution());
		}
	}

	public int isMax(double weight, double[] predictions) {
		int count = 0;
		for(int i = 0; i < predictions.length; i++) {
			if(predictions[i] > weight) {
				return 0;
			}
			else if(predictions[i] == weight) {
				count++;
			}
		}
		if(count >= 4) {
			return 0;
		}
		return 1;
	}


	public NeuralNetworkMixing getCopy() {
		return new NeuralNetworkMixing(_g,_seed,_options);
	}

	public String toString() {
		return "Neural Netowrk Mixing(\\nu = " + _learningRate + ") ";
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new NeuralNetworkMixing(g,seed,options);
	}
}