package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import brownstrategy.agent.Agent;

/**
 *
 * This oracle makes its predictions based on each agents
 * opponents moves.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.  This oracle only counts times that they
 * had the same utility (TOP,MID, or BOT).
 * 
 * @author jberg
 *
 */

public class PrevSeqOpponentsWithUtilOracle extends AbstractPrevSequenceOracle {

	final static int BOT = 0;
	final static int MID = 1;
	final static int TOP = 2;

	public PrevSeqOpponentsWithUtilOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> opp1,opp2;
		ArrayList<Integer> playerBeingModeled;
		if (opp == 0) {
			opp1 = player2Moves;
			opp2 = player3Moves;
			playerBeingModeled = player1Moves;
		}
		else if (opp == 1) {
			opp1 = player1Moves;
			opp2 = player3Moves;
			playerBeingModeled = player2Moves;
		}
		else {
			opp1 = player1Moves;
			opp2 = player2Moves;
			playerBeingModeled = player3Moves;
		}
		if(_numLastMoves == 0 || playerBeingModeled.size() <= _numLastMoves) {
			for (Integer move : playerBeingModeled) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			int[] movesArr;
			if (opp == 2) {
				movesArr = new int[]{player2Moves.get(playerBeingModeled.size() - 1),player1Moves.get(playerBeingModeled.size() - 1),player3Moves.get(playerBeingModeled.size() - 1)};
			}
			else {
				movesArr = new int[]{player3Moves.get(playerBeingModeled.size() - 1),player1Moves.get(playerBeingModeled.size() - 1),player2Moves.get(playerBeingModeled.size() - 1)};
			}
			int util = getUtilRank(movesArr);
			List<Integer> prevMoves1 = opp1.subList(opp1.size() - _numLastMoves, opp1.size());
			List<Integer> prevMoves2 = opp2.subList(opp2.size() - _numLastMoves, opp2.size());
			for(int i = 0; i < playerBeingModeled.size() - _numLastMoves; i++) {
				List<Integer> subMoves1 = opp1.subList(i, i+_numLastMoves);
				List<Integer> subMoves2 = opp2.subList(i, i+_numLastMoves);
				if (arrListEquals(prevMoves1, subMoves1) && arrListEquals(prevMoves2, subMoves2)) {
					int[] movesArr2;
					if (opp == 2) {
						movesArr2 = new int[]{player2Moves.get(i+_numLastMoves),player1Moves.get(i+_numLastMoves),player3Moves.get(i+_numLastMoves)};
					}
					else {
						movesArr2 = new int[]{player3Moves.get(i+_numLastMoves),player1Moves.get(i+_numLastMoves),player2Moves.get(i+_numLastMoves)};
					}
					int util2 = getUtilRank(movesArr2);
					if(util == util2) {
						arr[playerBeingModeled.get(i+_numLastMoves)] = arr[playerBeingModeled.get(i+_numLastMoves)] + 1;
					}
				}
			}
		}
		return arr;
	}

	public int getUtilRank(int[] moves) {
		int[] utils = new int[3];
		utils[0] = getUtility(moves);
		utils[1] = getUtility(moves[1],moves[0],moves[2]);
		utils[2] = getUtility(moves[2],moves[0],moves[1]);
		if(utils[0] >= utils[1]) {
			if(utils[0] >= utils[2]) {
				return TOP;
			}
			else {
				return MID;
			}
		}
		else if(utils[0] >= utils[2]) {
			return MID;
		}
		else {
			return BOT;
		}
	}

	public String toString() {
		return "PrevSeqOpponentsWithUtilOracle(n = " + _numLastMoves + ")";
	}

	public PrevSeqOpponentsWithUtilOracle getCopy() {
		return new PrevSeqOpponentsWithUtilOracle(_g,_seed,_options);
	}
	
	@Override
	public PrevSeqOpponentsWithUtilOracle getCopy(Game g, long seed, String options) {
		return new PrevSeqOpponentsWithUtilOracle(g,seed,options);
	}

}