package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * This oracle makes its predictions based on the moves
 * of all agents.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.
 * 
 * @author jberg
 *
 */

public class PrevSequenceAllOracle extends AbstractPrevSequenceOracle {

	public PrevSequenceAllOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> moves;
		if(opp == 0) {
			moves = player1Moves;
		}
		else if (opp == 1) {
			moves = player2Moves;
		}
		else {
			moves = player3Moves;
		}
		if(_numLastMoves == 0 || moves.size() <= _numLastMoves) {
			for (Integer move : moves) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			List<Integer> prevMoves1 = player1Moves.subList(player1Moves.size() - _numLastMoves, player1Moves.size());
			List<Integer> prevMoves2 = player2Moves.subList(player2Moves.size() - _numLastMoves, player2Moves.size());
			List<Integer> prevMoves3 = player3Moves.subList(player3Moves.size() - _numLastMoves, player3Moves.size());
			for(int i = 0; i < moves.size() - _numLastMoves; i++) {
				List<Integer> subMoves1 = player1Moves.subList(i, i+_numLastMoves);
				List<Integer> subMoves2 = player2Moves.subList(i, i+_numLastMoves);
				List<Integer> subMoves3 = player3Moves.subList(i, i+_numLastMoves);
				if (arrListEquals(prevMoves1, subMoves1) &&
						arrListEquals(prevMoves2, subMoves2) &&
						arrListEquals(prevMoves3, subMoves3)) {
					arr[moves.get(i+_numLastMoves)] = arr[moves.get(i+_numLastMoves)] + 1;
				}
			}
		}
		return arr;
	}
	
	public String toString() {
		return "PrevSequenceAllOracle(n = " + _numLastMoves + ")";
	}
	
	public PrevSequenceAllOracle getCopy() {
		return new PrevSequenceAllOracle(_g,_seed,_options);
	}
	
	public PrevSequenceAllOracle getCopy(Game g,long seed,String options) {
		return new PrevSequenceAllOracle(_g,_seed,_options);
	}
}