/*
Agent: Frozen Pontiac
Team: Prateek Tandon, Michael Wellman, Patrick Jordan
 (Computer Science & Engineering, University of Michigan)

Description/Methodology: 

We performed an empirical equilibrium analysis among the strategies 
provided in sample code, plus a few basic strategies of our own 
devising.  We then selected a subset based on this analysis, and used 
these as reference strategies for our agent Frozen Pontiac (FP).  FP 
employs a form of fictitious play, but with respect to these reference 
strategies, rather than stage-level actions.
*/

package edu.umich.FrozenPontiac;

import java.util.*;
import game.*;
import lemonade.*;
import strategy.*;
/**
 *
 * @author prateekt@umich.edu
 */

public class FrozenPontiac implements Strategy{

    int p1curr = 0;
    int p2curr = 0;
    int p1prev = 0;
    int p2prev = 0;
    int mycurr = 0;
    int myprev = 0;
    private Hashtable p1_hist = new Hashtable();
    private Hashtable p2_hist = new Hashtable();
    double p1constwt = 0.65;
    double p1ratwt = 0.14;
    double p1ermwt = 0.14;
    double p1n2pwt = 0.07;
    double p2constwt = 0.65;
    double p2ratwt = 0.14;
    double p2ermwt = 0.14;
    double p2n2pwt = 0.07;
    Game game;
    long gameseed;

    int p1guess_const = p1curr;
    int p2guess_const = p2curr;

    int p1guess_rat = p1curr;
    int p2guess_rat = p2curr;

    int p1guess_erm = p1curr;
    int p2guess_erm = p2curr;

    int p1guess_n2p = p1curr;
    int p2guess_n2p = p2curr;

    int stratlen = 4;
    int cnt;
    
    public FrozenPontiac (Game g, long seed, String options) {
        game = g;
        gameseed = seed;
        
        p1_hist.put("constant", p1constwt);
        p1_hist.put("rational", p1ratwt);
        p1_hist.put("erms", p1ermwt);
        p1_hist.put("n2p", p1n2pwt);

        p2_hist.put("constant", p2constwt);
        p2_hist.put("rational", p2ratwt);
        p2_hist.put("erms", p2ermwt);
        p2_hist.put("n2p", p2n2pwt);
  }


    public int fictitious_play()
    {
        boolean p1const = checkconst(1);
        boolean p2const = checkconst(2);
        boolean p1opp = checkopp(1);
        boolean p2opp = checkopp(2);
        boolean p1erm = checkerms(1);
        boolean p2erm = checkerms(2);
        boolean p1n2p = checkn2p(1);
        boolean p2n2p = checkn2p(2);
        double p1prob[] = new double[stratlen];
        double p2prob[] = new double[stratlen];

        if (p1const)
            p1_hist.put("constant", (Double)p1_hist.get("constant")+1);
        
        if (p2const)
            p2_hist.put("constant", (Double)p2_hist.get("constant")+1);

        if (p1opp)
            p1_hist.put("rational", (Double)p1_hist.get("rational")+1);
        
        if (p2opp)
            p2_hist.put("rational", (Double)p2_hist.get("rational")+1);
        
        if (p1erm)
            p1_hist.put("erms", (Double)p1_hist.get("erms")+1);
        
        if (p2erm)
            p2_hist.put("erms", (Double)p2_hist.get("erms")+1);

        if (p1n2p)
            p1_hist.put("n2p", (Double)p1_hist.get("n2p")+1);

        if (p2n2p)
            p2_hist.put("n2p", (Double)p2_hist.get("n2p")+1);


        if (!(p1const&&p1opp&&p1erm&&p1n2p))
        {
            p1_hist.put("constant", (Double)p1_hist.get("constant")+1);
            p1_hist.put("rational", (Double)p1_hist.get("rational")+1);
            p1_hist.put("erms", (Double)p1_hist.get("erms")+1);
            p1_hist.put("n2p", (Double)p1_hist.get("n2p")+1);
        }

        if (!(p2const&&p2opp&&p2erm&&p2n2p))
        {
            p2_hist.put("constant", (Double)p2_hist.get("constant")+1);
            p2_hist.put("rational", (Double)p2_hist.get("rational")+1);
            p2_hist.put("erms", (Double)p2_hist.get("erms")+1);
            p2_hist.put("n2p", (Double)p2_hist.get("n2p")+1);
        }
        
        Double div1 = (Double)p1_hist.get("constant")+(Double)p1_hist.get("rational")+(Double)p1_hist.get("erms")+(Double)p1_hist.get("n2p");
        Double div2 = (Double)p2_hist.get("constant")+(Double)p2_hist.get("rational")+(Double)p2_hist.get("erms")+(Double)p2_hist.get("n2p");


        p1prob[0] = ((Double)(p1_hist.get("constant")))/div1;
        p2prob[0] = ((Double)(p2_hist.get("constant")))/div2;

        p1prob[1] = ((Double)(p1_hist.get("rational")))/div1;
        p2prob[1] = ((Double)(p2_hist.get("rational")))/div2;

        p1prob[2] = ((Double)(p1_hist.get("erms")))/div1;
        p2prob[2] = ((Double)(p2_hist.get("erms")))/div2;

        p1prob[3] = ((Double)(p1_hist.get("n2p")))/div1;
        p2prob[3] = ((Double)(p2_hist.get("n2p")))/div2;

        p1guess_const = p1curr;
        p2guess_const = p2curr;

        p1guess_rat = rational(mycurr, p2curr);
        p2guess_rat = rational(mycurr, p1curr);

        p1guess_n2p = p1curr;
        p2guess_n2p = p2curr;

        int p1guess[] = new int[stratlen];
        int p2guess[] = new int[stratlen];
        
        p1guess[0] = p1guess_const;
        p1guess[1] = p1guess_rat;
        p1guess[2] = p1guess_erm;
        p1guess[3] = p1guess_n2p;

        p2guess[0] = p2guess_const;
        p2guess[1] = p2guess_rat;
        p2guess[2] = p2guess_erm;
        p2guess[3] = p2guess_n2p;
        
        double eu[] = new double[12];

        for (int i=0; i<12; i++)
        {
            eu[i] = calc_eu(i, p1prob, p2prob, p1guess, p2guess);
        }

        double maxu = 0;
        int maxidx = 0;
        for (int i=0; i<12; i++)
        {
            if (eu[i]>maxu)
            {
                maxu = eu[i];
                maxidx = i;
            }
            else if (eu[i]==maxu)
            {
                double r = Math.random();
                if (r<Math.random())
                    maxidx = i;
            }
        }
        return maxidx;
    }



  @Override
  public int getAction() {
      int mymove = fictitious_play();
      return mymove;
  }

  @Override
  public void observeOutcome(int[] actions) {
      p1prev = p1curr;
      p2prev = p2curr;
      myprev = mycurr;
      p1curr = actions[1];
      p2curr = actions[2];
      mycurr = actions[0];
  }

  public boolean checkconst(int pnum)
  {
      if (pnum==1)
      {
          if (p1curr==p1prev)
              return true;
      }
      if (pnum==2)
      {
          if (p2curr==p2prev)
              return true;
      }
      return false;
  }

  public boolean checkopp(int pnum)
  {
      if (pnum==1)
      {
        if(rational(myprev, p2prev)==p1curr)
            return true;
      }
      if (pnum==2)
      {
        if(rational(myprev, p1prev)==p2curr)
            return true;
      }
      return false;
  }

  public boolean checkn2p(int pnum)
  {
      if (pnum==1)
      {
          if((Math.abs(p2curr-p1curr)==1)||(Math.abs(mycurr-p1curr)==1))
            return true;
          else if ((p2curr==11&&p1curr==0)||(p2curr==0&&p1curr==11))
              return true;
          else if ((p1curr==11&&mycurr==0)||(p1curr==0&&mycurr==11))
              return true;
      }
      if (pnum==2)
      {
          if((Math.abs(p2curr-p1curr)==1)||(Math.abs(mycurr-p2curr)==1))
            return true;
          else if ((p2curr==11&&p1curr==0)||(p2curr==0&&p1curr==11))
              return true;
          else if ((p2curr==11&&mycurr==0)||(p2curr==0&&mycurr==11))
              return true;
      }
      return false;
  }

  //From sample code
  public boolean checkerms(int pnum)
  {
      if (pnum==1)
      {
          int temp[] = new int[3];
          int regrets[] = new int[12];
          Random r = new Random(gameseed);
          int[] possibleUtilities=new int[12];

          for(int i=0;i<possibleUtilities.length;i++)
          {
                temp[0]=i;
                possibleUtilities[i]=getutil(temp[0], myprev, p2prev)*6;
          }

          temp[0] = p1prev;
          temp[1] = myprev;
          temp[2] = p2prev;

          for(int i=0;i<possibleUtilities.length;i++)
          {
              regrets[i] += possibleUtilities[i]-possibleUtilities[temp[0]];
          }
          
          double total = 0;
          for(int i=0;i<regrets.length;i++)
          {
              if (regrets[i]>0) total+=regrets[i];
          }
          if (total>0)
          {
              double dart = r.nextDouble()*total;
              for(int i=0;i<regrets.length;i++)
              {
                  if (regrets[i]>0)
                  {
                      dart-=regrets[i];
                      if (dart<0)
                        p1guess_erm=i;
                  }
              }
           }
          if (p1guess_erm==p1curr)
            return true;
          
          p1guess_erm = r.nextInt(12);
       }
       
      if (pnum==2)
      {
          int temp[] = new int[3];
          int regrets[] = new int[12];
          Random r = new Random(gameseed);
          int[] possibleUtilities=new int[12];

          for(int i=0;i<possibleUtilities.length;i++)
          {
                temp[0]=i;
                possibleUtilities[i]=getutil(temp[0], myprev, p2prev)*6;
          }

          temp[0] = p2prev;
          temp[1] = myprev;
          temp[2] = p1prev;

          for(int i=0;i<possibleUtilities.length;i++)
          {
              regrets[i] += possibleUtilities[i]-possibleUtilities[temp[0]];
          }

          double total = 0;
          for(int i=0;i<regrets.length;i++)
          {
              if (regrets[i]>0) total+=regrets[i];
          }
          if (total>0)
          {
              double dart = r.nextDouble()*total;
              for(int i=0;i<regrets.length;i++)
              {
                  if (regrets[i]>0)
                  {
                      dart-=regrets[i];
                      if (dart<0)
                        p2guess_erm=i;
                  }
              }
           }
          if (p2guess_erm==p2curr)
            return true;

          p2guess_erm = r.nextInt(12);
       }
      return  false;
  }

  public int rational(int p1guess, int p2guess)
  {
    double outer_avg = 0;
    double inner_avg = 0;


    double opp_diff = Math.abs(p1guess - p2guess);
      if (opp_diff >= 6)
      {
          outer_avg = Math.min(p1guess, p2guess) + opp_diff/2;
          inner_avg = (outer_avg+6)%12;
      }
      else if (opp_diff!=0 && opp_diff<6)
      {
          inner_avg = Math.min(p1guess, p2guess) + opp_diff/2;
          outer_avg = (inner_avg+6)%12;
      }
      else if (opp_diff==0)
      {
          if (p1guess!=0)
          {
              inner_avg = p1guess;
              outer_avg = (inner_avg+6)%12;
          }
          else
          {
              outer_avg = 6;
              inner_avg = 0;
          }
      }
      return (int) outer_avg;
   }


   public double calc_eu(int p0, double p1pr[], double p2pr[], int[] p1, int[] p2)
   {
       double util = 0;
       for (int i=0; i<stratlen; i++)
       {
           for(int j=0; j<stratlen; j++)
           {
               util = util + p1pr[i]*p2pr[j]*getutil(p0, p1[i], p2[j]);
           }
       }
       return util;
   }


   public int getutil (int p0, int p1, int p2)
   {
       int util[] = new int[3];
       int actions[] = new int[3];
       actions[0] = p0;
       actions[1] = p1;
       actions[2] = p2;
       Utility l = new LemonadeUtility(12);
       util = l.getUtility(actions);
       return util[0]/6;
   }


}
