package strategy;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;
import lemonade.LemonadeUtility;
import strategy.Player;
import strategy.MyUtils;
import static java.lang.Math.*;

/* Author: Dustin Morrill
 * Date: September 2, 2010
 *
 * Agent description:
 * This is my "smartest" agent and is based on conversations Johnny and I had
 * over what the best balance between playing across from a constant player
 * and playing next to them.  It tries to be somewhat aggressive and initiate
 * a sandwich and will also look for opportunities to sandwich while playing
 * across from the constant player.  If the both players are playing constant,
 * this agent tries to move them from their alignment and tries to play the
 * constant game as well.
 *
 * One thing that should be improved on is its capacity to forgive agents that
 * appear constant.  Right now this agent does not forgive at all.
 *
 */

public class ualberta_morrill_ba_v3 implements Strategy {
    Utility util;
    MyUtils agent_utils;

    public ualberta_morrill_ba_v3 (Game game, long randomSeed, String options) {
        util = game.getUtility();
        agent_utils = new MyUtils(game.getNumPlayers(), game.getNumActions(), randomSeed);
        agent_utils.getMyPlayer().setPos(agent_utils.getRandomInt(agent_utils.getNumActions()));
    }

    private void setMyPos () {
        // If the other opponents are both defensive or one is random, play
        // defensive
        Player cp = agent_utils.getMostConstantOpponent();
        Player mp = agent_utils.getMyPlayer();
        //System.out.printf("most constant: %d\n", p.getLabel());
        Player[] random_opponents = agent_utils.getRandomOpponents();
        if (random_opponents.length == 2) {

        }
        else if(agent_utils.bothDefensive() || random_opponents.length == 1 ||
                (agent_utils.sharingOneSpace() && !agent_utils.inDesperationMode())) {
            agent_utils.setDefenseMode(cp);
        }
        // If I haven't moved yet and my last utility is lower than the number
        // of spaces on the board divided by 2 + 1, change to attack mode
        else if(mp.checkConstant(agent_utils.getRound() - 1)
                &&
            mp.getLatestUtil() < agent_utils.getNumActions() / 2 + 1) {
            agent_utils.setAttackMode(cp);
        }
        // If you are attacking, continue attacking for a few turns so the other
        // player has a chance to join in a sandwich
        else if (agent_utils.inAttackMode() && agent_utils.getNumRoundsInCurrMode() < agent_utils.getModellingThreshold()) {
            agent_utils.setAttackMode(agent_utils.getTarget());
        }
        // If the other player is attacking a constant player, target that
        // constant player and go into attack mode
        else if (agent_utils.opponentsAreTogether()) {
            agent_utils.setAttackMode(agent_utils.getMostConstantOpponent());
        }
        // If the other players are across from each other, go into desperation
        // mode
        else if (agent_utils.opponentsAreAcross()) {
            // If you were in attack mode before, the other opponent should be
            // targeted next
            Player target = new Player(-1);
            if (agent_utils.inAttackMode()) {
                target = agent_utils.getOtherOpponent();
            }
            else if (agent_utils.targetLocked()) {
                target = agent_utils.getTarget();
            }
            else {
                target = agent_utils.getMostConstantOpponent();
            }
            agent_utils.setDesperationMode(target);
        }
        else {
            /*else if (bothOffensive()) {
                setUniformMode();
            } To be implemented later*/
            // Otherwise play constant
        }
    }

    private int getMyPos () {
        return agent_utils.getMyPlayer().getLatestPos();
    }

// Game interface methods
    public int getAction () {
        return getMyPos();
    }
    
    public void observeOutcome (int[] actions) {
        agent_utils.updateRound();
        int round = agent_utils.getRound();
        int utils[] = util.getUtility(actions);
        int modelling_threshold = agent_utils.getModellingThreshold();
        int num_players = agent_utils.getNumPlayers();
        for (int i = 0; i < num_players; i++)
        {
            Player p = agent_utils.getPlayer(i);
            // Set positions from last round
            if (i != 0) {
                p.setPos(actions[i]);
            }
            // Set utilities from last round
            int curr_util = utils[i];
            p.setLatestUtil(curr_util);
            //System.out.printf("after set latest util: curr_util: %d, getLatestUtil: %d\n", curr_util, p.getLatestUtil());
            p.updateTotalUtil(curr_util);
            if (round > modelling_threshold-1) {
           //     p.setConstant(p.checkConstant(modelling_threshold-1));
            }
        }
        if (round > 1) {
            for (int i = 1; i < num_players; i++) {
                Player p = agent_utils.getPlayer(i);
                int pos = p.getLatestPos();
                // If you were attacking an opponent and the other opponent didn't
                // join in a sandwhich or move across from our target and it
                // moved to a new position, it's probably playing randomly
                // Need to examine the position last turn
                Player op = agent_utils.getPlayer(i == 1 ? 2 : 1);
                int op_pos = op.getPos(round-1);
                //System.out.println("inAttackMode(): " + inAttackMode() + " positionsAreTogether: " + positionsAreTogether(pos, op_pos) + " positionsAreAccros: " + positionsAreAcross(pos, op_pos) + " checkConstant: " + p.checkConstant(1));
                if (agent_utils.inAttackMode() && !agent_utils.positionsAreTogether(pos, op_pos) &&
                        !agent_utils.positionsAreAcross(pos, op_pos) && !p.checkConstant(1)) {
                    p.setRandom(true);
                    //System.out.println("Setting player " + p.getLabel() + " random");
                }
                // If you were attacking an opponent for a few turns and the other
                // opponent played across from where the opponent was last turn
                if (agent_utils.inAttackMode() &&
                        agent_utils.getNumRoundsInCurrMode() == modelling_threshold) {
                    agent_utils.getOtherOpponent().setDefensive();
                }
                // Need to assign agents as offensive here in some way
            }
        }
        // Update player rankings
        agent_utils.updateRankings();
        // Set the position for next round
        setMyPos();
    }
}

