package edu.inaoe;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.io.*;
import java.util.Date;
import java.text.SimpleDateFormat;

import edu.inaoe.util.LeaderFollowerDomain;
import edu.inaoe.util.OpponentModel;
import edu.inaoe.util.Strategy;
import edu.inaoe.util.StrategyState;


import strategy.UtilityBasedStrategy;

import game.Game;

/**
 * @author Enrique Munoz de Cote
 * 
 */

public class TeamUP extends UtilityBasedStrategy {

	private Vector<Integer> sequence = new  Vector<Integer>();
	private LeaderFollowerDomain stratDomain;
	//states transforms from joint actions to its encoding state
	private Map<Vector<Strategy>,StrategyState> states = new HashMap<Vector<Strategy>, StrategyState>();
	//this is the algorithm's current high level strategy
	private Map<StrategyState,Strategy> strategy = new HashMap<StrategyState, Strategy>();
	//given a state, returns a set of pairs <action,value>
	Map<StrategyState,Map<Strategy,Double>> Q = new HashMap<StrategyState,Map<Strategy,Double>>();
	//Map<StrategyState,Double> V = new HashMap<StrategyState,Double>(stratDomain.size());
	private int currentAction = 6;
	private Strategy currentStrategy;
	StrategyState lastState;
	StrategyState currentState;
	private OpponentModel opponents[] = new OpponentModel[2];
	private int timeStep;
	private File file;
	private File tempFile;
	private boolean log = true;
	StringBuffer content;
	private int K = 15;
	//for computing followIndex
	private static final double GAMMA = 0.05;
	private static final double RHO = 0.5;
	//shaping
	private boolean shapingOn = true;
	int[] totals = new int[3];
	int[][] c= new int [2][4];
	int[] stateCount = new int [65];
	
	//private static final int T = 2; //how many time steps to stick to current strategy
	/**
	 * @param g
	 * @param seed
	 */
	public TeamUP(Game g, long seed, String options) {
		super(g, seed);
		if(!options.isEmpty())
			setParams(options);
		initLog();
		timeStep = 0;
		for (int i = 0; i < g.getNumPlayers()-1; i++) {
			opponents[i] = new OpponentModel(12);
		}
		stratDomain = new LeaderFollowerDomain(opponents);
		StrategyState s0 = new StrategyState();
		stratDomain.add(s0);
		currentStrategy = new Strategy("S");
		for (StrategyState state : stratDomain.getStateSet()) {
			if(shapingOn){
				if(!shape(state, g.getNumPlayers(),s0))
					state.Init(g.getNumPlayers(), stratDomain.getPlayerStrategySet("1"), s0, 6, K);
			}else state.Init(g.getNumPlayers(), stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			states.put(state.getFeatures(),state);
		}
		lastState = s0;
		ValueIteration();
	}
	
	  @Override
	  public void observeOutcome(int[] actions){
	    currentAction = actions[0];
	    for (int i = 0; i < opponents.length; i++) {
			opponents[i].lastAction(actions[i+1], actions[0]);
		}
	    //System.out.println(actions[0]+","+actions[1]+","+actions[2]);
	    Vector<Strategy> jointStrat = computeState();
	    //content.append(jointStrat+ "\r");
	    int[] possibleUtilities = getPossibleUtilities(actions);
	    if (update(jointStrat,possibleUtilities[currentAction])){    
	    	ValueIteration();
	    	//System.out.println(": " + strategy.get(currentState));
	    	//content.append("+++ update for state: "+currentState.getFeatures()+ strategy.get(currentState)+"\n");
	    	//strattoLog();
	    }
		currentStrategy = strategy.get(currentState);
		computeAction(currentStrategy);
		//System.out.println("ACTION("+currentAction+"):"+Q.get(currentState).get(currentStrategy));
	    sequence.add(currentAction);
	    
		timeStep++;
	  }
	  
	  /**
	   * does the requied updates
	 * @param jointStrat current joint strategy
	 * @param r instantaneous reward
	 * @return true if replanning is required
	 */
	public boolean update(Vector<Strategy> jointStrat, int r){
		  boolean update = false;
		  currentState = states.get(jointStrat);
		  //currentState.update(currentStrategy(currentState), r);
		  //if(lastState != currentState)
			//  System.out.println(currentState.getFeatures());
		  update = lastState.update(strategy.get(lastState), currentState, r);
		  lastState = currentState;
		  return update;
	  }
	  
	  public void computeAction(Strategy strategy){
		if(strategy.getName() == "F2")
			currentAction = opposite(0);
		else if (strategy.getName() == "F3")
			currentAction = opposite(1);
	  }
	  
		public int opposite(int i){
			int opp = ( (opponents[i].currentAction() + 6) % 12 );
			if (opp == 12)
				opp = 0;
			return opp;
		}
	  public void ValueIteration(){
		  final double EPSILON = 0.01; 
		  final double GAMMA = 0.5;
		  double error = 12;
		  Set<Strategy> actions = stratDomain.getPlayerStrategySet("1");
		  
		  Map<StrategyState,Double> V = new HashMap<StrategyState,Double>(stratDomain.size());
		  //Map<StrategyState,Map<Strategy,Double>> Q = new HashMap<StrategyState,Map<Strategy,Double>>();
		  
		  for (StrategyState state : stratDomain.getStateSet()) {
			  V.put(state, 0.0);
			  Map<Strategy,Double>Qaux = new HashMap<Strategy,Double>();
			  
			  for (Strategy action : actions) 
				  Qaux.put(action, 0.0);
			  Q.put(state, Qaux);
		  }		
		  
		  int counter = 0;
		  while (error > EPSILON) {
		  //while (counter< 50) {
		  // (int  counter=0; error>EPSILON;counter++) {
			  error = -9999;
			  //int c=0;
			  for (StrategyState state : stratDomain.getStateSet()) {
				  double Vstate = -9999;
				  //int c1=0;
				  for (Strategy action : actions) {
					  Map<StrategyState,Double> probs = state.probs(action);
					  //System.out.println(probs.size());
					  //expected over future states
					  double sum = 0;
					  //System.out.println(probs.size());
					  for (StrategyState state1 : probs.keySet()) {
						sum += probs.get(state1) * V.get(state1);
					  }
					  double total = state.getReward(action) + GAMMA * sum;
					  Q.get(state).put(action, total);
					  if(total > Vstate){
						  Vstate = total;
						  strategy.put(state, action);
					  }
					  ///c1++;
				  }
				  error = Math.max(error, Math.abs(Vstate - V.get(state)));
				  V.put(state, Vstate);
				  //c++;
				  //System.out.println("+++"+c);
			  }	
			  counter++;
			  //System.out.println(counter);
		  }
		  
		  //System.out.println(counter);
		  //System.out.println(strattoString());
		  
	  }
	  
	  public Strategy currentStrategy(StrategyState state){
		  double curMax = -9999;
		  double tmpMax = -9999;
		  Strategy curStrat = null;
		  for (Strategy strat : stratDomain.getPlayerStrategySet("1")) {
			tmpMax = Math.max(curMax, Q.get(state).get(strat));
			if(tmpMax>curMax){
				curStrat = strat;
				curMax = tmpMax;
			}
		  }
		  return curStrat;
	  }
	 
	/* (non-Javadoc)
	 * @see strategy.UtilityBasedStrategy#observeUtilities(int[], int)
	 */
	@Override
	public void observeUtilities(int[] possibleUtilities, int currentAction) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see game.Strategy#getAction()
	 */
	@Override
	public int getAction() {
		return currentAction;
	}
	
	/**
	 * THIS FUNCTION IS THE ONE TO BE PLAYED WITH 
	 * up to now the strategy sticks if somebody is in front of it
	 * or follows agent 1
	 * @return the return type is dummy
	 */
	private Vector<Strategy> computeState(){
		Vector<Strategy> jointStrat = new Vector<Strategy>();
		jointStrat.add(currentStrategy);
		
		/*double f1 = followIndex(opponents[0].getSequence());
		//System.out.println("followIndex:"+ fOp+" threshold:"+opponents[i].threashold() );
		double f2 = followIndex(opponents[1].getSequence());
		//System.out.println("followIndex:"+ fOp+" threshold:"+opponents[i].threashold() );
		if(f1 < TRESHOLD)
			jointStrat.add(new Strategy("F1"));
		else if(f2 < TRESHOLD)
		jointStrat.add(new Strategy("F2"));*/
		
		for (int i = 0; i < opponents.length; i++) {
			//System.out.println("+++ "+ (i+1) +" +++");
			int j;
			if(i == 0) j =1;
			else j = 0;
			Strategy strat;
			double fOp = opponents[i].followIndex(opponents[j].getSequence());
			//System.out.println("followIndex:"+ fOp+" threshold:"+opponents[i].threashold() );
			double fMe = opponents[i].followIndex(sequence);
			//System.out.println("followIndex:"+ fOp+" threshold:"+opponents[i].threashold() );
			if(fMe < opponents[i].threashold()){
				strat = new Strategy("F1");
				c[i][0]++;
			}
			else if(fOp < opponents[i].threashold()){
				strat = new Strategy("F"+(j+2));
				c[i][1]++;
			}
			else if(opponents[i].isSticking()){
				strat = new Strategy("S");
				c[i][2]++;
			}
			else{
				strat = new Strategy("O");
				c[i][3]++;
			}
			//System.out.println("strat"+strat.getName() );
			jointStrat.add(strat);
		}
		return jointStrat;
	}
	
	private boolean shape(StrategyState state, int n, StrategyState s0){
		Strategy f1 = new Strategy("F1");
		Strategy f2 = new Strategy("F2");
		Strategy f3 = new Strategy("F3");
		Strategy s = new Strategy("S");
		Vector<Strategy> st = new Vector<Strategy>(3);
		//optimal states
		st.add(f2); st.add(f1); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(f3); st.add(f1); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(f2); st.add(f1); st.add(s);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(f3); st.add(s); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(s); st.add(f1); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(s); st.add(f1); st.add(s);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		st.clear(); st.add(s); st.add(s); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 12, K);
			return true;
		}
		//worst states
		st.clear(); st.add(f2); st.add(f1); st.add(f2);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 0, K);
			return true;
		}
		st.clear(); st.add(f2); st.add(f3); st.add(f2);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 0, K);
			return true;
		}
		st.clear(); st.add(f3); st.add(f3); st.add(f2);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 0, K);
			return true;
		}
		st.clear(); st.add(f3); st.add(f3); st.add(f1);
		if(state.getFeatures().equals(st)){
			state.Init(n, stratDomain.getPlayerStrategySet("1"), s0, 0, K);
			return true;
		}
		return false;
		
	}
	
	/**
	 * computes the followIndex of a given sequence
	 * @param opponentSeq the sequence to compare to
	 * @return the normalized followIndex of the its sequence and the argument's sequence
	 * @TODO: work out the co-domain (range) of the function 
	 */
	public double followIndex(Vector<Integer> mySeq, Vector<Integer> otherSeq){
		double fIndex = 0;
		double sumG = 0;
		double dif;
		double dif1;
		double dif2;
		double norm;
		int t = 0;
		assert(this.sequence.size() == mySeq.size() && this.sequence.size() == otherSeq.size());
		for (int i = 0; i < sequence.size(); i++) 
			sumG += Math.pow(GAMMA, i);
		
		for(int i = sequence.size()-1; i >0 ; i--) {
			dif1 = Math.abs((sequence.get(i) -  mySeq.get(i-1) + 6) % 12);
			if(dif1 > 6)
				dif1 = 12 - dif1;
			dif2 = Math.abs((sequence.get(i) - otherSeq.get(i-1) + 6) % 12);
			if(dif2 > 6)
				dif2 = 12 - dif2;
			dif = Math.min(dif1, dif2);
			//if(t==0) System.out.print(dif+": ");
			dif = Math.pow(dif, RHO);
			norm = Math.pow(GAMMA, t) / sumG;
			fIndex += norm * dif;
			t++;
		}
		//System.out.println(fIndex);
		return fIndex;
	}
	
	/**
	 * computes the followIndex of a given sequence for a given opponent
	 * @param opponentSeq the sequence to compare to
	 * @return the normalized followIndex of the its sequence and the argument's sequence
	 * @TODO: work out the co-domain (range) of the function 
	 */
	public double followIndex(Vector<Integer> otherSeq){
		double fIndex = 0;
		double sumG = 0;
		double dif;
		double norm;
		int t = 0;
		assert(this.sequence.size() ==  otherSeq.size());
		for (int i = 0; i < sequence.size(); i++) 
			sumG += Math.pow(GAMMA, i);
		
		for(int i = sequence.size()-1; i >0 ; i--) {
			dif = Math.abs((sequence.get(i) - otherSeq.get(i-1) + 6) % 12);
			if(dif > 6)
				dif = 12 - dif;
			//if(t==0) System.out.print(dif+": ");
			dif = Math.pow(dif, RHO);
			norm = Math.pow(GAMMA, t) / sumG;
			fIndex += norm * dif;
			t++;
		}
		//System.out.println(fIndex);
		return fIndex;
	}
	
	private void initLog(){
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyMMddHHmm");
        Date date = new Date();
        File tmpPath = new File(System.getProperty("user.home")+"/experiments/lemonade/tmp/");
        String tempPath = System.getProperty("user.home")+"/experiments/lemonade/tmp/tmp.txt";
        String logPath = System.getProperty("user.home")+"/experiments/lemonade/tmp/log.txt";
        //create elog file or reuse if an elog file is already present
/*        File temp = new File(System.getProperty("user.home")+"/experiments/lemonade/tmp/");
        FilenameFilter filter = new FilenameFilter() { public boolean accept(File dir, String name) { return name.endsWith("elog"); } }; 
        String s[] = temp.list(filter); 
        
        if(s.length == 0)
        	file = new File(System.getProperty("user.home")+"/experiments/lemonade/tmp/"+dateFormat.format(date)+".elog");
        else if (s.length == 1) {
			file = new File(System.getProperty("user.home")+"/experiments/lemonade/tmp/" + s[0]);
		}
        else
        	assert(s.length <2);*/
        if(!tmpPath.exists()){
        	tmpPath.mkdirs();
        }
        file = new File(logPath);
		tempFile = new File(tempPath);
		getFileStateExperience(tempFile);
		content = new StringBuffer();
	}
	
	private void getFileStateExperience(File aFile){
	    if(aFile.exists()){
	    try {
	      //use buffering, reading one line at a time
	      //FileReader always assumes default encoding is OK!
	      BufferedReader input =  new BufferedReader(new FileReader(aFile));
	      try {
	        String line = null; //not declared within while loop
	        /*
	        * readLine is a bit quirky :
	        * it returns the content of a line MINUS the newline.
	        * it returns null only for the END of the stream.
	        * it returns an empty String if two newlines appear in a row.
	        */
	        int i=0;
	        int k=0;
	        while (( line = input.readLine()) != null){
	        	stateCount[i] += Integer.parseInt(line);
	        	k += stateCount[i];
	        	i++;
	        }
	        assert(k == 100);
	      }
	      finally {
	        input.close();
	      }
	    }
	    catch (IOException ex){
	      ex.printStackTrace();
	    }
	    }

	}
	public void getStatesExperience(int[] states){
		int i = 0;
		for (StrategyState state : stratDomain.getStateSet()) {
			states[i] += state.getExperience();
			i++;
		}
	}
	
	public void flushtoFile() throws FileNotFoundException, IOException {
		String ret =	System.getProperty("line.separator");
		int i = 0;
		content.setLength(0);
		for (StrategyState state : stratDomain.getStateSet()) {
			if(state.getFeatures().size()==0)
				content.append("S_0,");
			else{
			for (Strategy strat : state.getFeatures()) 
				content.append(strat.getName() + ",");
			}
			content.append("\t" + stateCount[i] + ret);
			i++;
		}
		
		//for (Integer c : totals) 
		//	content.append(c.toString()+" ");
		Writer output = new BufferedWriter(new FileWriter(file));
		    try {
		      //FileWriter always assumes default encoding is OK!
		      output.write( content.toString() );
		    }
		    finally {
		      output.close();
		    }
	}
	
	private void flushtoTempFile() throws FileNotFoundException, IOException {
		String ret =	System.getProperty("line.separator");
		for (Integer c : stateCount) 
			content.append(c.toString()+ret);
		  Writer output = new BufferedWriter(new FileWriter(tempFile));
		    try {
		      //FileWriter always assumes default encoding is OK!
		      output.write( content.toString() );
		    }
		    finally {
		      output.close();
		    }
	}
	
	private void updateStateExperience(){
		int i = 0;
		for (StrategyState state : stratDomain.getStateSet()) {
			stateCount[i] += state.getExperience();
			i++;
		}
	}
	
	private void strattoLog(){
		String ret =	System.getProperty("line.separator");
    	for (StrategyState state : strategy.keySet()) {
			content.append(""+state.getFeatures()+": ");
			content.append(""+strategy.get(state)+ret);	
		}
	}
	
	private StringBuffer strattoString(){
		StringBuffer cont = new StringBuffer();
		String ret =	System.getProperty("line.separator");
    	for (StrategyState state : strategy.keySet()) {
			cont.append(""+state.getFeatures()+": ");
			cont.append(""+strategy.get(state)+ret);	
		}
    	return cont;
	}

	
	public void observeUtility(int[] u) {
		//strattoLog();
		updateStateExperience();
	    for (int i=0;i<u.length;i++) {
			totals[i] += u[i];
		}
	    if(log){
		    try {
				flushtoTempFile();
				flushtoFile();
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }
	}
	
	private void setParams(String options){
		int index1 = 0;
		int index2 = 0;
		String sub;
		index1=options.indexOf(':');
		while (index1 != -1)
		{
			char key = options.charAt(index1-1);//gets the char after the :
			switch (key) {
			case '-':
				log = false;
				index2 = options.indexOf(':', index1+1);
				if (index2>0)
					sub = options.substring(index1+1, index2-1);
				else 
					sub = options.substring(index1+1);
				index1 = options.indexOf(':', index2+1);
				break;
			case 'K':
				index2 = options.indexOf(':', index1+1);
				if (index2>0)
					sub = options.substring(index1+1, index2-1);
				else 
					sub = options.substring(index1+1);
				this.K = Integer.parseInt(sub);
				index1 = options.indexOf(':', index2+1);
				break;
				
			case 'L':
				index2 = options.indexOf(':', index1+1);
				if (index2>0) 
					sub = options.substring(index1+1, index2-1);
				else
					sub = options.substring(index1+1);
				//tempFileName = sub;
				index1 = options.indexOf(':', index2+1);
				break;
			default:
				break;
			}
		}
	}

}