package edu.cmu.waugh;

import game.Game;
import game.Strategy;
import game.Utility;

import java.lang.Math;
import java.util.Arrays;
import java.util.Random;

public class Waugh implements Strategy {
    
    final double GAMMA = .2;

    Game game;
    Random rng;
    Utility utility;
    int n, p, k, action;
    int[] results, m;
    double[][] freq;
    double[] ent;
    double ev;
    
    public Waugh(Game game_, long seed, String options) {
	game    = game_;
	rng     = new Random(seed);
	utility = game.getUtility();
	p       = game.getNumPlayers();
	n       = game.getNumActions();
	
	action  = rng.nextInt(n);
	k       = 0;
	results = new int[p];
	m       = new int[p-1];
	freq    = new double[p-1][];
	ent     = new double[p-1];
	ev      = 0;
	for(int i=0; i<p-1; ++i) {
	    m[i] = game.getNumActions(i+1);

	    freq[i] = new double[m[i]];
	    Arrays.fill(freq[i], 1.0);

	    ent[i] = -Math.log(1./m[i]);
	}
    }

    public int getAction() {
	return action;
    }

    public void observeOutcome(int[] actions) {
	utility.getUtility(actions, results);
	ev += (results[0]-ev)/(++k);
	for(int i=0; i<p-1; ++i) {
	    double u = freq[i][actions[i+1]]/(k-1+m[i]);
	    freq[i][actions[i+1]] += 1.0;
	    double v = freq[i][actions[i+1]]/(k+m[i]);
	    ent[i] += -v*Math.log(v) + u*Math.log(u);
	}
	if (k > 10 && ev < 7 && rng.nextDouble() < GAMMA) {
	    double best = ent[0];
	    int besti   = 0;
	    for(int i=1; i<p-1; ++i) {
		if (ent[i] < best) {
		    best  = ent[i];
		    besti = i;
		}
	    }
	    action = (actions[besti+1]+n/2)%n;
	}
    }

}