package strategy;

import java.util.*;
import java.lang.*;

import game.Game;
import game.Strategy;
import lemonade.GeneralizedLemonadeUtility;


public class NewAcrossS implements Strategy{
	
	int numActions;
	int nextAction;
	int[] lastTurn;
	int sandwichPartner;
	int sandwichCount;
	int badCount;
	Integer[] acrossPositions;
	int[] acrossMap;
	int[] bestPositions;	
	GeneralizedLemonadeUtility utility;
	Random r;
	
	
	public NewAcrossS(Game g, long seed, String options) {
		this.r = new Random(seed);
		this.numActions = g.getNumActions();
		this.utility = (GeneralizedLemonadeUtility)g.getUtility();	

		this.sandwichPartner = 0;
		
		this.initAcrossPositions();
		this.initBestPositions();
				
		this.blindChooseMove();
	}
	
	//choose best across position. if no across positions, choose best position.
	private	void blindChooseMove(){
		if (acrossPositions.length > 0) {
			int bestMove = 0;
			int bestQual = 0;
			for (int i=0; i<acrossPositions.length; i++) {
				int quality	= this.getQuality(acrossPositions[i]);
				if (quality > bestQual) {
					bestMove = acrossPositions[i];
					bestQual = quality;
				}
			}
			this.nextAction = bestMove;
		} else {
			this.nextAction = bestPositions[this.r.nextInt(this.bestPositions.length)];
		}		
	}
	
	//find all positions where two players are guaranteed to do at least as well as the third
	private void initAcrossPositions(){
		Set<Integer> acrossSet = new HashSet<Integer>();
		this.acrossMap = new int[this.numActions]; 
		for (int i=0; i<this.numActions; i++) {
			for (int j=0; j<this.numActions; j++) {
				boolean collab = true;
				for (int k=0; k<this.numActions; k++) {
					int[] scores = this.utility.getUtility(new int[]{i,j,k});
					if (scores[0]<scores[2] || scores[1]<scores[2] || i==j) {
						collab = false;
						break;
					}
				}
				if (collab) {
					acrossSet.add(i);
					acrossSet.add(j);
					this.acrossMap[i] = j;
					this.acrossMap[j] = i;
				}
			}
		}
		acrossPositions = acrossSet.toArray(new Integer[acrossSet.size()]);
	}
	
	//modified from FancyConstantStrategy2
	//weights nearer locations more heavily
	protected int getQuality(int action){
				
		int oppositeAction = (action+utility.getIslandSize()/2)% utility.getIslandSize();
		int sideAction = (action+utility.getIslandSize()/4)% utility.getIslandSize();
		int sideAction2 = (action+3*utility.getIslandSize()/4)% utility.getIslandSize();

		int[] util = utility.getUtility(new int[]{action,oppositeAction,oppositeAction});
		int[] util2 = utility.getUtility(new int[]{action,sideAction,sideAction2});
		
		return util[0]+2*util2[0];
	}
	
	//from FancyConstantStrategy2
	private void initBestPositions(){
		int[] quality = new int[utility.getIslandSize()];
		for(int i=0;i<quality.length;i++){
			quality[i]=getQuality(i);
		}
		int maxDensity = quality[0];
		int maxDensityCount = 1;
		for(int i=0;i<quality.length;i++){
			
			if (maxDensity<quality[i]){
				maxDensity=quality[i];
				maxDensityCount=1;
			} else if (maxDensity==quality[i]){
				maxDensityCount++;
			}
		}
		this.bestPositions = new int[maxDensityCount];
		int bestActionIndex = 0;
		for(int i=0;i<quality.length;i++){
			if (quality[i]==maxDensity){
				this.bestPositions[bestActionIndex]=i;
				bestActionIndex++;
			}
		}
	}
	
	protected int bestResponse(int[] actions){
		int bestMove = 0;
		int bestUtil = 0;
		for (int i=0; i<this.numActions; i++) {
			int[] thisUtil = utility.getUtility(new int[]{i,actions[1],actions[2]});
			if (thisUtil[0] > bestUtil) {
				bestMove = i;
				bestUtil = thisUtil[0];
			}
		}
		return	bestMove;
	}
	
	private boolean sandwichCheck(int[] actions,int[] scores){
		if (scores[0]>scores[1] || scores[0]>scores[2] || r.nextInt(2)==0) {
			return false;
		}
		if (this.sandwichPartner != 0) {
			int sandwichTarget = 3 - this.sandwichPartner; //1->2, 2->1
			if (actions[this.sandwichPartner] == (actions[sandwichTarget]+1) % this.numActions) {
				this.nextAction = (actions[sandwichTarget]-1+this.numActions) % this.numActions;
				return true;
			}
			if (actions[this.sandwichPartner] == (actions[sandwichTarget]-1+this.numActions) % this.numActions) {
				this.nextAction = (actions[sandwichTarget]+1) % this.numActions;
				return true;
			}
			return false;
		}
		if (actions[2] == this.lastTurn[2] && actions[1] == ((actions[2]+1) % this.numActions)) {
			this.nextAction = (actions[2]-1+this.numActions) % this.numActions;
			this.sandwichPartner = 1;
			return true;
		}
		if (actions[2] == this.lastTurn[2] && actions[1] == ((actions[2]-1+this.numActions) % this.numActions)) {
			this.nextAction = (actions[2]+1) % this.numActions;
			this.sandwichPartner = 1;
			return true;
		}
		if (actions[1] == this.lastTurn[1] && actions[2] == ((actions[1]+1) % this.numActions)) {
			this.nextAction = (actions[1]-1+this.numActions) % this.numActions;
			this.sandwichPartner = 2;
			return true;
		}
		if (actions[1] == this.lastTurn[1] && actions[2] == ((actions[1]-1+this.numActions) % this.numActions)) {
			this.nextAction = (actions[1]+1) % this.numActions;
			this.sandwichPartner = 2;
			return true;
		}
		return false;
	}
	
	
	//choose best across partner. if no possible partner, play best response.
	private void reactChooseMove(int[] actions,int[] scores){
		int[] utils = new int[numActions];
		for (int i=0; i < this.acrossPositions.length ; i++) {
			if (actions[1] == acrossPositions[i]) {
				int[] thisUtil = utility.getUtility(new int[]{acrossMap[actions[1]],actions[1],actions[2]});
				utils[acrossPositions[i]] = thisUtil[0];
			}
			if (actions[2] == acrossPositions[i]) {
				int[] thisUtil = utility.getUtility(new int[]{acrossMap[actions[2]],actions[1],actions[2]});
				utils[acrossPositions[i]] = thisUtil[0];
			}
		}
		
		int bestAcross = 0;
		int bestUtil = 0;
		for (int i=0; i<this.numActions; i++) {
			if (utils[i] > bestUtil) {
				bestAcross = acrossMap[i];
				bestUtil = utils[i];
			}
		}
		if (bestUtil > scores[0]) {
			this.nextAction = bestAcross;
		} else {
			this.nextAction = bestResponse(actions);
		}
	}
	
	//if in last, react with p=1/2. otherwise stick.
	public void observeOutcome(int[] actions) {
		this.lastTurn = actions;
		int[] scores = this.utility.getUtility(actions);
		
		//check for sandwich offers to accept if in last
		if (this.sandwichCheck(actions,scores)) {
			sandwichCount += 1;
			this.badCount = 0;
		} else if (scores[0]<=scores[1] && scores[0]<=scores[2]) {
			this.badCount += 1;
			if (this.badCount > 0 && this.r.nextInt(2)==0) {
				reactChooseMove(actions,scores);
			}
		} else {
			this.badCount = 0;
		}

	}
	
	public int getAction() {
		return this.nextAction;
	}

}
