package human;

import java.util.Vector;

/**
 * An encapsulation for the model data. Synchronizes getting and setting of the model.
 * 
 * It is also possible to have a "write-lock" on the model.
 * @author maz
 *
 */
public class HumanModel {
  private HumanModelData data;
  
  private boolean lock;
  private Vector<HumanModelListener> listeners;
  
  public HumanModel(int numPlayers){
    data = new HumanModelData(numPlayers);
    listeners = new Vector<HumanModelListener>();
  }
  
  public void addListener(HumanModelListener listener){
    listeners.add(listener);
  }
  
  
  public synchronized void setModel(HumanModelData data){
    this.data.set(data);
    for(HumanModelListener listener:listeners){
      listener.handleChange(this);
    }
  }
  
  public synchronized void getModel(HumanModelData output){
    output.set(data);
  }
  
  public synchronized boolean attemptToLock(){
    if (lock){
      return false;
    }
    lock = true;
    return true;
  }
  
  public synchronized void releaseLock(){
    lock = false;
  }
  
  
  public void lockAndGetModel(HumanModelData output) throws InterruptedException{
    while(!attemptToLock()){
      Thread.sleep(2000);
    }
    getModel(output);
  }
  
  public void setModelAndUnlock(HumanModelData input){
    setModel(input);
    releaseLock();
  }
  
}
