package net;

import human.HumanModelData;

import java.applet.AppletStub;
import java.awt.AWTException;
import java.awt.Frame;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.SocketException;
import java.net.URL;


public class LemonadeAppletTest {

  public static void checkMessage(LineSocket socket,String expected) throws SocketException, IOException{
    String message = socket.receiveMessage();
    if (!message.equals(expected)){
      throw new RuntimeException("Expected "+expected+" Received "+message);
    }
  }
  
  
  public static void checkMessageState(HumanRemoteClient client, String expected){
    HumanModelData data = new HumanModelData(3);
    
    client.model.getModel(data);
    if (!expected.equals(data.message)){
      throw new RuntimeException("Expected:"+expected+" actual:"+data.message);
    }
  }
  
  /**
   * This moves the mouse to the proper coordinates and clicks the
   * button. Works only on my machine.
   * @param r
   */
  public static void clickQuit(Robot r){
    r.mouseMove(689,418);
    r.mousePress(InputEvent.BUTTON1_MASK);
    r.mouseRelease(InputEvent.BUTTON1_MASK);
  }

  /**
   * This moves the mouse to the proper coordinates and clicks the
   * button. Works only on my machine.
   * @param r
   */
  public static void clickPlayAgain(Robot r){
    r.mouseMove(597,413);
    r.mousePress(InputEvent.BUTTON1_MASK);
    r.mouseRelease(InputEvent.BUTTON1_MASK);
    
  }

  public static void checkHandshake(LineSocket link, ServerSocket server, HumanRemoteClient client) throws IOException, InterruptedException{
    link.openConnection(server.accept());
    
    // This crazy wait is because in a previous version, the code would
    // change to "Choose an action..." before the game had begun.
    Thread.sleep(6000);
    checkMessageState(client,"Waiting for other players...");
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    link.sendMessage("BEGIN:LEMONADE 3 12");
    Thread.sleep(1000);
    checkMessageState(client,"Begin playing!");
  }
  
  public static void main(String[] args) throws IOException, InterruptedException, AWTException{
    LemonadeApplet applet = new LemonadeApplet();
    
    URL codebase = new URL("file:/C:/Documents%20and%20Settings/maz/workspace/Lemonade/bin/");
    codebase.getHost();
    AppletStub stub = new LemonadeAppletStub(applet,codebase);
    applet.setStub(stub);
    applet.init();
    
    Frame f = new Frame("Applet Frame");
    
    f.add(applet);
    
    f.pack();
    f.validate();
    f.setVisible(true);
    
    HumanRemoteClient client=applet.client;
    
    //Game g = new Game(new int[] { 12, 12, 12 }, new ScaledLemonadeUtility(12));
    //HumanStrategy constantStrategy = new HumanStrategy(g, 0, "");
    //HumanRemoteClient client = new HumanRemoteClient(constantStrategy,g);
    
    ServerSocket server = new ServerSocket(5000);
    
    applet.start();
    Thread.sleep(1000);
    
    checkMessageState(client,"Waiting for other players...");
    
    LineSocket link = new LineSocket("SERVER");
    link.openConnection(server.accept());
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    checkMessageState(client,"Waiting for other players...");
    
    link.sendMessage("BEGIN:1 LEMONADE 3 12");
    
    Thread.sleep(1000);
    checkMessageState(client,"Begin playing!");
    
    
    client.strategy.clickOnLocation(3);
    
    checkMessage(link,"3");
    checkMessageState(client,"Action locked in...");
    Thread.sleep(1000);
    
    
    link.sendMessage("CHAT:HELLO");
    
    
    Thread.sleep(1000);
    //Thread.sleep(7000);
    //checkMessageState(client,"Choose an action...");
    
    
    
    
    
    
    applet.stop();
    
    
    checkMessage(link,"12");
    System.err.println("Message correctly received");
    
    applet.start();
    
    link.openConnection(server.accept());
    
    Thread.sleep(1000);
    checkMessageState(client,"Waiting for other players...");
    
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    
    link.sendMessage("BEGIN:1 LEMONADE 3 12");
    
    Thread.sleep(1000);
    checkMessageState(client,"Begin playing!");
    
    System.err.println("Test completed");
    applet.stop();
    f.dispose();
    
  }
}
