package net;

import game.Game;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.Vector;

import lemonade.ScaledLemonadeUtility;


/**
 * 
 * @author maz
 *
 */
public class Room implements Runnable {
  ServerSocket server;
  Vector<Socket> holdingPattern;
  Vector<GameServer> gameServers;
  long gameIndex;
  Game g;
  String logDirectory;
  boolean active;
  
  int numActiveUsers;
  int numInactiveUsers;
  
  int numRounds;
  long receivingTimeout;
  
  public Room(int port, int numGameServers, Game g, String logLocation, int numRounds, long timeout) throws IOException{
    this.logDirectory = logLocation;
    this.numRounds = numRounds;
    this.receivingTimeout = timeout;
    server = new ServerSocket(port);
    holdingPattern = new Vector<Socket>();
    gameServers = new Vector<GameServer>();
    this.g = g;
    numActiveUsers = 0;
    numInactiveUsers = 0;
    active = true;
    initializeGameIndex();
  }
  
  
  private void initializeGameIndex() {
    gameIndex = 0;
  }

  public synchronized int getNumActiveUsers(){
    int total = 0;
    for(GameServer server:gameServers){
      total+=server.getNumUsers();
    }
    return total;
  }


  public void updateStats(){
    numInactiveUsers = getNumInactiveUsers();
    numActiveUsers = getNumActiveUsers();
  }
  public synchronized int getNumInactiveUsers(){
    return holdingPattern.size();
  }

  public synchronized int getNumUsers(){
    updateStats();
    return numActiveUsers+numInactiveUsers;
  }
  
  public String toString(){
    String result = "Inactive users:"+getNumInactiveUsers();
    for(int i=0;i<holdingPattern.size();i++){
      Socket s = holdingPattern.get(i);
      result+=" "+s.getInetAddress()+":"+s.getPort();
    }
    result+="\n";
    for(int i=0;i<gameServers.size();i++){
      GameServer gameServer = gameServers.get(i);
      if (gameServer.getNumUsers()>0){
        result+=gameServer.toString()+"\n";
      }
    }
    return result;
  }
  
  public synchronized void startGame(){
    long currentGameIndex = ++gameIndex;
    GameServer server = new GameServer(g,currentGameIndex,logDirectory+"game",numRounds,receivingTimeout);
    gameServers.add(server);
    
    for(int i=0;i<g.getNumPlayers();i++){
      Socket s2 = holdingPattern.get(0);
      holdingPattern.removeElementAt(0);
      server.addClient(s2);
      
    }
    server.start();
  }
  
  public void run() {
    try {
    while(active){
      server.setSoTimeout(10000);
      updateStats();
      System.out.println("Server listening on port "+server.getLocalPort()+" Active:"+numActiveUsers +" Inactive:"+numInactiveUsers);
      System.out.println(this.toString());
      try{
      Socket s  = server.accept();
      holdingPattern.add(s);
      if (holdingPattern.size()>=g.getNumPlayers()){
        startGame();
      }
      } catch (SocketTimeoutException ste){
        
      }
      
    }
    } catch (IOException io){
      System.err.println("IOException occurred in main server thread");
      io.printStackTrace();
    } 

  }
  
  
  public static void main(String[] args) throws IOException{
    Game g= new Game(new int[]{12,12,12},new ScaledLemonadeUtility(12));
    int numGameServers = 10;
    int port = 5000;
    Room r = new Room(port, numGameServers, g,"",20,60000);
    r.run();
  }
}
