package stats;

import util.Array;
import game.Game;
import game.GameGenerator;
import game.Utility;


/**
 * An observer of utility, like jointactionutility, that does
 * not keep track of joint actions, simply utilities.
 * @author maz
 *
 */
public class UtilitySeriesObserver implements SeriesObserver{
  /**
   * The utility function.
   */
  Utility utility;
  
  
  /**
   * 
   */
  long[] gameUtilitySquares;
  
  /**
   * 
   */
  int[] seriesTotals;
  
  
  /**
   * The total utilities up until now.
   */
  int[] gameUtilityTotals;
  
  
  /**
   * The last recorded utility.
   */
  int[] lastUtility;
  
  
  public UtilitySeriesObserver(GameGenerator g){
  }
  /**
   * Construct a utility observer.
   * @param utility a utility function to be used.
   * @param numPlayers the number of players in the game.
   */
  public UtilitySeriesObserver(Utility utility, int numPlayers){
    this.utility = utility;
    gameUtilityTotals = new int[numPlayers];
    lastUtility=new int[numPlayers];
    seriesTotals = new int[numPlayers];
    gameUtilitySquares = new long[numPlayers];
  }
  
  private void init(Utility utility, int numPlayers){
    this.utility = utility;
    gameUtilityTotals = new int[numPlayers];
    lastUtility=new int[numPlayers];
    seriesTotals = new int[numPlayers];
    gameUtilitySquares = new long[numPlayers];
  }
  public void observeOutcome(int[] actions) {
    utility.getUtility(actions,lastUtility);
    for(int i=0;i<lastUtility.length;i++){
      gameUtilityTotals[i]+=lastUtility[i];
    }
  }
  
  /**
   * Display the utilities for the series and the squares of the utilities.
   */
  public void display(String[] playerNames) {
    System.out.println(Array.arrayToNakedString(playerNames));
    System.out.println(Array.arrayToNakedString(seriesTotals));
    System.out.println(Array.arrayToNakedString(gameUtilitySquares));
    
  }
  public int[] getUtilities() {
    return gameUtilityTotals;
  }

  public int[] getSeriesTotals() {
    return seriesTotals;
  }
  
  public long[] getGameUtilitySquares() {
    return gameUtilitySquares;
  }
  

  public void finishGame() {
    
    for(int i=0;i<gameUtilityTotals.length;i++){
      seriesTotals[i]+=gameUtilityTotals[i];
      gameUtilitySquares[i]+=(gameUtilityTotals[i]*gameUtilityTotals[i]);
      gameUtilityTotals[i]=0;
    }
  }

  public void finishSeries() {
  }
  @Override
  public void startGame(Game g) {
    if (utility==null){
      this.init(utility, g.getNumPlayers());
    }
    utility = g.getUtility();
    
    // TODO Auto-generated method stub
    
  }
}
