package stats.collusion;


import game.Game;
import game.Utility;

public class UtilityCollusionObserver extends AbstractCollusionObserver{

  public int[] utilityInCollusion;
  public int[] utilityAsSucker;
  public int[] roundsInCollusion;
  public int[] roundsAsSucker;
  public int[] roundsNotInCollusion;
  public int[] utilityNotInCollusion;
  
  
  int[] currentCollusionUtility;
  
  Utility utility;
  
  @Override
  public void observeTerminateCollusion(boolean countBroken) {
    int length = getCollusionLength();
    if (collusionDidNotCount()){
      for(int i=0;i<currentCollusionUtility.length;i++){
        utilityNotInCollusion[i]+=currentCollusionUtility[i];
        roundsNotInCollusion[i]+=length;
      }
    } else {
      int player0 = this.collusionPlayers[0];
      int player1 = this.collusionPlayers[1];
      int sucker = this.getSucker();
      utilityInCollusion[player0]+=currentCollusionUtility[player0];
      utilityInCollusion[player1]+=currentCollusionUtility[player1];
      utilityAsSucker[sucker]+=currentCollusionUtility[sucker];
      roundsInCollusion[player0]+=length;
      roundsInCollusion[player1]+=length;
      roundsAsSucker[sucker]+=length;
    }
    currentCollusionUtility = null;
  }

  @Override
  public void observeOutcome(int[] actions){
    super.observeOutcome(actions);
    if (!wasInCollusion()){
      incrementNotInCollusionUtility();
    }
  }

  public UtilityCollusionObserver(Game g) {
    super(g);
    this.utility = g.getUtility();
    int numPlayers = g.getNumPlayers();
    assert(numPlayers==3);
    utilityInCollusion = new int[numPlayers];
    utilityAsSucker = new int[numPlayers];
    utilityNotInCollusion = new int[numPlayers];
    roundsInCollusion = new int[numPlayers];
    roundsAsSucker = new int[numPlayers];
    roundsNotInCollusion = new int[numPlayers];
    MIN_LENGTH=1;
  }

  @Override
  public void observeContinueCollusion() {
    incrementCollusionUtility();
  }


  private void incrementNotInCollusionUtility(){
    int[] util = utility.getUtility(actions);
    for(int i=0;i<utilityNotInCollusion.length;i++){
      utilityNotInCollusion[i]+=util[i];
      roundsNotInCollusion[i]++;
    }
  }
  
  
  private void incrementCollusionUtility(){
    int[] util = utility.getUtility(actions);
    for(int i=0;i<currentCollusionUtility.length;i++){
      currentCollusionUtility[i]+=util[i];
    }
  }
  @Override
  public void observeStartCollusion() {
    currentCollusionUtility = new int[utilityInCollusion.length];
    incrementCollusionUtility();
  }
  @Override
  public void display(String[] playerNames) {
    assert(playerNames.length==3);
    for(int i=0;i<playerNames.length;i++){
      String data = "{\"Player\":"+playerNames[i] + "," 
      +"\"all\":[" + playerNames[0]+","+playerNames[1]+","+playerNames[2]+"],"
      +"\"utilityInCollusion\":"+utilityInCollusion[i]+","
      +"\"utilityAsSucker\":"+utilityAsSucker[i]+","
      +"\"utilityNotInCollusion\":"+utilityNotInCollusion[i]+","
      +"\"roundsInCollusion\":"+roundsInCollusion[i]+","
      +"\"roundsAsSucker\":"+roundsAsSucker[i]+","
      +"\"roundsNotInCollusion\":"+roundsNotInCollusion[i] + "}";
      System.out.println(data);
    }
  }

  @Override
  public void startGame(Game g) {
    utility = g.getUtility();
    
  }

}
