package strategy;

import game.Game;



/**
 * The external regret matching strategy is based on Blackwell's approachability
 * theorem.
 * @author maz
 *
 */
public class ExternalRegretMatchingStrategy extends UtilityBasedStrategy{

  /**
   * The total regret of not playing each algorithm.
   */
  int[] regrets;
  
  /**
   * Construct an external regret matching strategy based on
   * Blackwell's approachability theorem.
   * @param g
   * @param seed
   */
  public ExternalRegretMatchingStrategy(Game g, long seed, String options) {
    super(g,seed);
    this.regrets = new int[numActions];
  }
  
  public int[] getRegrets(){
    return regrets;
  }

  public void setRegrets(int[] vector){
    assert(regrets.length==vector.length);
    regrets=vector;
  }

  public int getAction() {
    //System.err.println("Regret matching strategy"+Array.arrayToString(regrets));
    double total = 0;
    for(int i=0;i<regrets.length;i++){
      
      if (regrets[i]>0){
        total+=regrets[i];
      }
    }
    if (total>0){
      double dart = r.nextDouble()*total;
      for(int i=0;i<regrets.length;i++){
        if (regrets[i]>0){
          dart-=regrets[i];
          if (dart<0){
            return i;
          }
        }
      }
    }
    return r.nextInt(numActions);
  }

  
  @Override
  public void observeUtilities(int[] possibleUtilities,int actionTaken) {
    for(int i=0;i<possibleUtilities.length;i++){
      regrets[i] += possibleUtilities[i]-possibleUtilities[actionTaken];
    }
  }
  
  
  
}
