package strategy;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Vector;

import lemonade.GeneralizedLemonadeUtility;



/**
 * A strategy that chooses a random action on the first round, and then
 * plays that action thereafter.
 * @author maz
 *
 */
public class FancyConstantStrategy2 implements Strategy{
  /**
   * The action the player plays
   */
  int constantAction;
  
  int[] bestActions;
  
  GeneralizedLemonadeUtility utility;
  
  
  
  protected int getQuality(int action){
    int oppositeAction = (action+utility.getIslandSize()/2)% utility.getIslandSize();
    int[] util = utility.getUtility(new int[]{action,oppositeAction,oppositeAction});
    return util[0];
  }
  
  private void initBestActions(){
    int[] quality = new int[utility.getIslandSize()];
    for(int i=0;i<quality.length;i++){
      quality[i]=getQuality(i);
    }
    int maxDensity = quality[0];
    int maxDensityCount = 1;
    for(int i=0;i<quality.length;i++){
      
      if (maxDensity<quality[i]){
        maxDensity=quality[i];
        maxDensityCount=1;
      } else if (maxDensity==quality[i]){
        maxDensityCount++;
      }
    }
    bestActions = new int[maxDensityCount];
    int bestActionIndex = 0;
    for(int i=0;i<quality.length;i++){
      if (quality[i]==maxDensity){
        bestActions[bestActionIndex]=i;
        bestActionIndex++;
      }
    }
  }
  /**
   * Construct a constant strategy. The game is
   * used to get the number of actions, and the
   * seed is used to seed a pseudorandom number 
   * generator, which then is queried for the action 
   * to take.
   * 
   * @param g
   * @param seed
   */
  public FancyConstantStrategy2(Game g, long seed, String options) {
    Random r = new Random(seed);
    this.utility = (GeneralizedLemonadeUtility)g.getUtility();
    
    initBestActions();
    constantAction = bestActions[r.nextInt(bestActions.length)];
  }
  



  public int getAction() {
    return constantAction;
  }

  public void observeOutcome(int[] actions) {
  }
  
  
  
}
