package com.yahoo.lsg.lifelong;

import static org.junit.Assert.*;

import java.lang.reflect.InvocationTargetException;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.yahoo.lsg.lifelong.GameInformation;
import com.yahoo.lsg.lifelong.PlayerRecord;

public class PlayerRecordTest {

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testPlayerRecord() throws SecurityException, NoSuchMethodException {
    try{
    new PlayerRecord("abc","xyz");
    fail("Should throw a ClassNotFoundException");
    } catch (ClassNotFoundException e){
      
    }
  }


  @Test
  public void testPlayerRecord2() throws SecurityException, ClassNotFoundException, NoSuchMethodException {
    PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    assertNotNull(record);
  }

  @Test
  public void testInitialize() throws SecurityException, ClassNotFoundException, NoSuchMethodException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException {
    PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    GameInformation info = new GameInformation(100, 200, 300);
    record.initialize(info, 0);
    assertNotNull(record.strategy);
  }

  @Test
  public void testIncrementUtility() throws SecurityException, ClassNotFoundException, NoSuchMethodException {
    final PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    record.incrementUtility(1.0);
    record.incrementUtility(2.0);
    record.endLifetime();
    
    final double mean = record.lifetimeUtility.getMean();
    assertEquals(1.5,mean,0.0);
  }

  @Test
  public void testEndLifetime() throws SecurityException, ClassNotFoundException, NoSuchMethodException {
    final PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    record.incrementUtility(1.0);
    record.incrementUtility(2.0);
    record.endLifetime();
    record.incrementUtility(3.0);
    record.incrementUtility(4.0);
    record.endLifetime();
    
    final double mean = record.lifetimeUtility.getMean();
    
    assertEquals(2.5,mean,0.0);
  }

  @Test
  public void testEndLifetime2() throws SecurityException, ClassNotFoundException, NoSuchMethodException {
    final PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    record.incrementUtility(1.0);
    record.incrementUtility(2.0);
    record.endLifetime();
    record.endLifetime();
    
    final double mean = record.lifetimeUtility.getMean();
    
    assertEquals(0.75,mean,0.0);
  }



  @Test
  public void testCompareTo() throws SecurityException, ClassNotFoundException, NoSuchMethodException {
    final PlayerRecord record = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    record.incrementUtility(1.0);
    record.incrementUtility(2.0);
    record.endLifetime();
    record.incrementUtility(3.0);
    record.incrementUtility(4.0);
    record.endLifetime();

    final PlayerRecord record2 = new PlayerRecord("com.yahoo.lifelong.examples.ConstantLifelongStrategy","");
    record2.incrementUtility(-1.0);
    record2.incrementUtility(-2.0);
    record2.endLifetime();
    record2.incrementUtility(-3.0);
    record2.incrementUtility(-4.0);
    record2.endLifetime();
    
    final int resultGreaterThan = record.compareTo(record2);
    assertTrue(resultGreaterThan>0);
    assertEquals(0,record.compareTo(record));
  }

}
