package stats.collusion;

import static org.junit.Assert.*;

import java.util.Random;

import game.Game;
import lemonade.ScaledLemonadeUtility;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class UtilityCollusionObserverTest {
  UtilityCollusionObserver observer;
  @Before
  public void setUp() throws Exception {
    int numActions = 12;
    ScaledLemonadeUtility utility = new ScaledLemonadeUtility(numActions);
    Game g = new Game(new int[]{numActions,numActions,numActions},utility);
    observer = new UtilityCollusionObserver(g);
  }

  
  @After
  public void tearDown() throws Exception {
    observer = null;
  }

  @Test
  public void testHolisticCollusion(){
    int[] actions = new int[]{0,6,9};
    for(int i=0;i<100;i++){
      observer.observeOutcome(actions);
    }
    observer.finishGame();
    observer.finishSeries();
    assertEquals(100,observer.roundsInCollusion[0]);
    assertEquals(100,observer.roundsInCollusion[1]);
    assertEquals(0,observer.roundsInCollusion[2]);
    assertEquals(0,observer.roundsNotInCollusion[0]);
    assertEquals(0,observer.roundsNotInCollusion[1]);
    assertEquals(0,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(0,observer.roundsAsSucker[1]);
    assertEquals(100,observer.roundsAsSucker[2]);
  }

  @Test
  public void testHolisticNotInCollusion(){
    int[] actions = new int[]{0,4,8};
    for(int i=0;i<100;i++){
      observer.observeOutcome(actions);
    }
    observer.finishGame();
    observer.finishSeries();
    assertEquals(0,observer.roundsInCollusion[0]);
    assertEquals(0,observer.roundsInCollusion[1]);
    assertEquals(0,observer.roundsInCollusion[2]);
    assertEquals(100,observer.roundsNotInCollusion[0]);
    assertEquals(100,observer.roundsNotInCollusion[1]);
    assertEquals(100,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(0,observer.roundsAsSucker[1]);
    assertEquals(0,observer.roundsAsSucker[2]);
  }

  

  @Test
  public void testHolisticSwitch(){
    int[] actionsNotInCollusion = new int[]{0,4,8};
    int[] actionsInCollusion = new int[]{0,6,9};
    
    for(int i=0;i<50;i++){
      observer.observeOutcome(actionsInCollusion);
    }
    for(int i=0;i<50;i++){
      observer.observeOutcome(actionsNotInCollusion);
    }
    
    observer.finishGame();
    observer.finishSeries();
    assertEquals(50,observer.roundsInCollusion[0]);
    assertEquals(50,observer.roundsInCollusion[1]);
    assertEquals(0,observer.roundsInCollusion[2]);
    assertEquals(50,observer.roundsNotInCollusion[0]);
    assertEquals(50,observer.roundsNotInCollusion[1]);
    assertEquals(50,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(0,observer.roundsAsSucker[1]);
    assertEquals(50,observer.roundsAsSucker[2]);
  }

  @Test
  public void testSwitchBetween(){
    int[] actionsInCollusion02 = new int[]{0,9,6};
    int[] actionsInCollusion01 = new int[]{0,6,9};
    
    for(int i=0;i<50;i++){
      observer.observeOutcome(actionsInCollusion01);
    }
    for(int i=0;i<50;i++){
      observer.observeOutcome(actionsInCollusion02);
    }
    
    observer.finishGame();
    observer.finishSeries();
    assertEquals(100,observer.roundsInCollusion[0]);
    assertEquals(50,observer.roundsInCollusion[1]);
    assertEquals(50,observer.roundsInCollusion[2]);
    assertEquals(0,observer.roundsNotInCollusion[0]);
    assertEquals(0,observer.roundsNotInCollusion[1]);
    assertEquals(0,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(50,observer.roundsAsSucker[1]);
    assertEquals(50,observer.roundsAsSucker[2]);
  }

  @Test
  public void testSwitchBetween3(){
    int[] actionsInCollusion02 = new int[]{0,9,6};
    int[] actionsInCollusion01 = new int[]{0,6,9};
    int[] actionsNotInCollusion = new int[]{0,4,8};
    int[][] choices = new int[][]{
        actionsInCollusion02,
        actionsInCollusion01,
        actionsNotInCollusion
        };
    int[] currentActions = actionsInCollusion01;
    Random r = new Random(0);
    for(int round=0;round<100;round++){
      observer.observeOutcome(currentActions);
      
      if (r.nextInt(10)==0){
        int index = r.nextInt(3);
        currentActions = choices[index];
      }
    }
    observer.finishGame();
    observer.finishSeries();
    
    int[] totalRounds = new int[3];
    for(int player=0;player<3;player++){
      totalRounds[player]=observer.roundsInCollusion[player]+observer.roundsNotInCollusion[player]+observer.roundsAsSucker[player];
    }
    assertEquals(100,totalRounds[0]);
    assertEquals(100,totalRounds[1]);
    assertEquals(100,totalRounds[2]);
    
  }

  @Test
  public void testRealGame(){
    int[][] actions = new int[][]{
        {1,6,10},
        {1,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {6,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0},
        {0,6,0}
    };
    for(int round=0;round<100;round++){
      observer.observeOutcome(actions[round]);
    }
    observer.finishGame();
    observer.finishSeries();
    
    int[] totalRounds = new int[3];
    for(int player=0;player<3;player++){
      totalRounds[player]=observer.roundsInCollusion[player]+observer.roundsNotInCollusion[player]+observer.roundsAsSucker[player];
    }
    assertEquals(100,totalRounds[0]);
    assertEquals(100,totalRounds[1]);
    assertEquals(100,totalRounds[2]);
    
  }
  @Test
  public void testSwitchBetween2(){
    int[] actionsInCollusion02 = new int[]{0,9,6};
    int[] actionsInCollusion01 = new int[]{0,6,9};
    
    for(int i=0;i<5;i++){
      observer.observeOutcome(actionsInCollusion01);
    }
    for(int i=0;i<5;i++){
      observer.observeOutcome(actionsInCollusion02);
    }
    
    observer.finishGame();
    observer.finishSeries();
    assertEquals(10,observer.roundsInCollusion[0]);
    assertEquals(5,observer.roundsInCollusion[1]);
    assertEquals(5,observer.roundsInCollusion[2]);
    assertEquals(0,observer.roundsNotInCollusion[0]);
    assertEquals(0,observer.roundsNotInCollusion[1]);
    assertEquals(0,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(5,observer.roundsAsSucker[1]);
    assertEquals(5,observer.roundsAsSucker[2]);
  }

  @Test
  public void testHolisticShortCollusion(){
    int[] actionsNotInCollusion = new int[]{0,4,8};
    int[] actionsInCollusion = new int[]{0,6,9};
    
    for(int i=0;i<2;i++){
      observer.observeOutcome(actionsInCollusion);
    }
    for(int i=0;i<98;i++){
      observer.observeOutcome(actionsNotInCollusion);
    }
    
    observer.finishGame();
    observer.finishSeries();

    assertEquals(0,observer.roundsInCollusion[0]);
    assertEquals(0,observer.roundsInCollusion[1]);
    assertEquals(0,observer.roundsInCollusion[2]);
    assertEquals(100,observer.roundsNotInCollusion[0]);
    assertEquals(100,observer.roundsNotInCollusion[1]);
    assertEquals(100,observer.roundsNotInCollusion[2]);
    assertEquals(0,observer.roundsAsSucker[0]);
    assertEquals(0,observer.roundsAsSucker[1]);
    assertEquals(0,observer.roundsAsSucker[2]);
  }
}
